==> lapisworks-1.5.6.8-sources.jar: com/luxof/lapisworks/VAULT/VAULT.java <==
package com.luxof.lapisworks.VAULT;

import at.petrak.hexcasting.api.casting.eval.CastingEnvironment;

import com.luxof.lapisworks.items.shit.InventoryItem;

import java.util.function.BiConsumer;
import java.util.function.Predicate;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_2960;
import net.minecraft.class_3222;
import org.jetbrains.annotations.Nullable;

/** Very Ass and Unnecessary but not use-Less Terminal (for items)
 * <p>Fetches, drains, gives items.
 * <p>Dynamic. Does not need to save into NBT data or anything.
 * <p>Adding your own type support if you want (why would you?) is easy.
 * <p>Right now, only god and I know why these math formulas make sense.
 * Soon, only god will know.
 */
public abstract class VAULT {
    protected VAULT() {}
    // this is the part you can mixin to add support
    @Nullable
    public static <ANY extends Object> VAULT of(ANY literallyFuckingAnything) {
        if (literallyFuckingAnything instanceof class_3222 player) {
            return new ServPlayerVAULT(player);
        } else if (literallyFuckingAnything instanceof CastingEnvironment castEnv) {
            return new CastEnvVAULT(castEnv);
        } else {
            return null;
        }
    }


    public abstract class_2960 getKindOfVault();


    protected int handleFetchInvItem(class_1799 stack, Predicate<class_1792> searchingFor, Flags flags) {
        return stack.method_7909() instanceof InventoryItem invItem && invItem.canAccess(flags) ?
                invItem.fetch(stack, searchingFor) : 0;
    }
    protected int handleFetchStack(class_1799 stack, Predicate<class_1792> searchingFor, Flags flags) {
        return searchingFor.test(stack.method_7909()) ? stack.method_7947() : 0;
    }
    /** Returns the amount of that item present. */
    public abstract int fetch(Predicate<class_1792> itemPred, Flags flags);
    /** Returns the amount of that item present. */
    public int fetch(class_1792 item, Flags flags) { return this.fetch(it -> it == item, flags); };


    protected int handleDrainInvItem(class_1799 stack, Predicate<class_1792> take, int amount, Flags flags) {
        if (!(stack.method_7909() instanceof InventoryItem invItem && invItem.canAccess(flags))) return amount;
        return invItem.drain(stack, take, amount);
    }
    protected <ANY extends Object> int handleDrainStack(
        class_1799 stack,
        Predicate<class_1792> take,
        int amount,
        ANY firstArg,
        BiConsumer<ANY, class_1799> setStackFunc
    ) {
        if (!take.test(stack.method_7909())) return amount;
        int leftInStack = Math.max(stack.method_7947() - amount, 0);
        int leftToTake = amount - (stack.method_7947() - leftInStack);
        setStackFunc.accept(
            firstArg,
            leftInStack == 0 ? class_1799.field_8037.method_7972() : new class_1799(stack.method_7909(), leftInStack)
        );
        return leftToTake;
    }
    /** Returns the amount that couldn't be drained. */
    public abstract int drain(Predicate<class_1792> itemPred, int amount, Flags flags);
    /** Returns the amount that couldn't be drained. */
    public int drain(class_1792 item, int amount, Flags flags) { return this.drain(it -> it == item, amount, flags); };


    protected int handleGiveInvItem(class_1799 stack, Predicate<class_1792> give, int amount, Flags flags) {
        if (!(stack.method_7909() instanceof InventoryItem invItem && invItem.canAccess(flags))) return amount;
        return invItem.give(stack, give, amount);
    }
    protected <ANY extends Object> int handleGiveStack(
        class_1799 stack,
        Predicate<class_1792> give,
        int amount,
        ANY firstArg,
        BiConsumer<ANY, class_1799> setStackFunc
    ) {
        if (!give.test(stack.method_7909())) return amount;
        int limit = stack.method_7914() - stack.method_7947();
        int left = Math.max(amount - limit, 0);
        setStackFunc.accept(
            firstArg,
            new class_1799(stack.method_7909(), Math.min(stack.method_7947() + amount, stack.method_7914()))
        );
        return left;
    }
    /** Returns the amount that couldn't be given. */
    public abstract int give(Predicate<class_1792> itemPred, int amount, Flags flags);
    /** Returns the amount that couldn't be given. */
    public int give(class_1792 item, int amount, Flags flags) { return this.give(it -> it == item, amount, flags); };
}

==> lapisworks-1.5.7-sources.jar: com/luxof/lapisworks/VAULT/VAULT.java <==
package com.luxof.lapisworks.VAULT;

import at.petrak.hexcasting.api.casting.eval.CastingEnvironment;

import com.luxof.lapisworks.items.shit.InventoryItem;

import java.util.function.BiConsumer;
import java.util.function.Predicate;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_2960;
import net.minecraft.class_3222;
import org.jetbrains.annotations.Nullable;

/** Very Ass and Unnecessary but not use-Less Terminal (for items)
 * <p>Fetches, drains, gives items.
 * <p>Dynamic. Does not need to save into NBT data or anything.
 * <p>Adding your own type support if you want (why would you?) is easy.
 * <p>Right now, only god and I know how this code works.
 * Soon, only god will know.
 */
public abstract class VAULT {
    protected VAULT() {}
    // this is the part you can mixin to add support
    /** if you want a VAULT, use ((GetVAULT)thething).grabVAULT(), not this.
     * This is meant to be stored in classes via mixin so you can grab a VAULT. */
    @Nullable
    public static <ANY extends Object> VAULT of(ANY literallyFuckingAnything) {
        if (literallyFuckingAnything instanceof class_3222 player) {
            return new ServPlayerVAULT(player);
        } else if (literallyFuckingAnything instanceof CastingEnvironment castEnv) {
            return new CastEnvVAULT(castEnv);
        } else {
            return null;
        }
    }


    public abstract class_2960 getKindOfVault();


    protected int handleFetchInvItem(class_1799 stack, Predicate<class_1792> searchingFor, Flags flags) {
        return stack.method_7909() instanceof InventoryItem invItem && invItem.canAccess(flags) ?
                invItem.fetch(stack, searchingFor) : 0;
    }
    protected int handleFetchStack(class_1799 stack, Predicate<class_1792> searchingFor, Flags flags) {
        return searchingFor.test(stack.method_7909()) ? stack.method_7947() : 0;
    }
    /** Returns the amount of that item present. */
    public abstract int fetch(Predicate<class_1792> itemPred, Flags flags);
    /** Returns the amount of that item present. */
    public int fetch(class_1792 item, Flags flags) { return this.fetch(it -> it == item, flags); };


    protected int handleDrainInvItem(class_1799 stack, Predicate<class_1792> take, int amount, Flags flags) {
        if (!(stack.method_7909() instanceof InventoryItem invItem && invItem.canAccess(flags))) return amount;
        return invItem.drain(stack, take, amount);
    }
    protected <ANY extends Object> int handleDrainStack(
        class_1799 stack,
        Predicate<class_1792> take,
        int amount,
        ANY firstArg,
        BiConsumer<ANY, class_1799> setStackFunc
    ) {
        if (!take.test(stack.method_7909())) return amount;
        int leftInStack = Math.max(stack.method_7947() - amount, 0);
        int leftToTake = amount - (stack.method_7947() - leftInStack);
        setStackFunc.accept(
            firstArg,
            leftInStack == 0 ? class_1799.field_8037.method_7972() : new class_1799(stack.method_7909(), leftInStack)
        );
        return leftToTake;
    }
    /** Returns the amount that couldn't be drained. */
    public abstract int drain(Predicate<class_1792> itemPred, int amount, Flags flags);
    /** Returns the amount that couldn't be drained. */
    public int drain(class_1792 item, int amount, Flags flags) { return this.drain(it -> it == item, amount, flags); };


    protected int handleGiveInvItem(class_1799 stack, Predicate<class_1792> give, int amount, Flags flags) {
        if (!(stack.method_7909() instanceof InventoryItem invItem && invItem.canAccess(flags))) return amount;
        return invItem.give(stack, give, amount);
    }
    protected <ANY extends Object> int handleGiveStack(
        class_1799 stack,
        Predicate<class_1792> give,
        int amount,
        ANY firstArg,
        BiConsumer<ANY, class_1799> setStackFunc
    ) {
        if (!give.test(stack.method_7909())) return amount;
        int limit = stack.method_7914() - stack.method_7947();
        int left = Math.max(amount - limit, 0);
        setStackFunc.accept(
            firstArg,
            new class_1799(stack.method_7909(), Math.min(stack.method_7947() + amount, stack.method_7914()))
        );
        return left;
    }
    /** Returns the amount that couldn't be given. */
    public abstract int give(Predicate<class_1792> itemPred, int amount, Flags flags);
    /** Returns the amount that couldn't be given. */
    public int give(class_1792 item, int amount, Flags flags) { return this.give(it -> it == item, amount, flags); };
}
